#! /usr/users/goldnbrg/bin/perl
# NOTE: modify the previous line to point to 
# the appropriate perl for your system


# A perl script to create binned data for a 2-d histogram
# and output data as input file for Igor
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!



# reads an input file from standard input  with paired data
# and an optional file with Boltzman weights
# generates a matrix with elements representing
# counts of pairs within grid space
# accepts one input argument, which is the
# name of a file with parameters:

#  	name of optional file with weights. Use "0" for no weights 
#       (don't use file names that start with "0"!)
#       Title for X axis (first column)
#  	number of bins for data in first column (X)
#  	min value of range for data in first column (X)
#  	max value of range for data in first column (X)
#       Title for Y axis (second column)
#  	number of bins for data in second column (Y)
#  	min value of range for data in second column (Y)
#  	max value of range for data in second column (Y)
#       optional lines with x-y pairs (space delimited) to be used as ref points


# writes data matrix to stdout 
# data are normalized to represent percent of total population

# Data are output so that row position represents X value (1st column in data file)
# and column positions represent Y value (2nd column in data file)
# order of rows is from minimum to maximum.
# order of columns is minimum to maximum.

# output includes Igor commands to generate an image plot.  
# Read the resulting file into Igor with the Load Igor Text command 
# in the Data menu.


# Reference points specified on the optional lines at the end of the parameter
# file are used to set bin values to maximum of those calculated for data.
# Be careful with this!

$parFile = $ARGV[0];

open (PARFILE,$parFile);

#read optional weight file 
$fileLine = <PARFILE>;
chop $fileLine;

if ($fileLine !~ /^0/) {
    $useWeights = 1;
    $weightFile = $fileLine;
} else {
    $useWeights = 0;
}


# read parameters
$xTitle = <PARFILE>;
chop $xTitle;
$binCtX = <PARFILE>;
chop $binCtX;
$minX = <PARFILE>;
chop $minX;
$maxX = <PARFILE>;
chop $maxX;
$yTitle = <PARFILE>;
chop $yTitle;
$binCtY = <PARFILE>;
chop $binCtY;
$minY = <PARFILE>;
chop $minY;
$maxY = <PARFILE>;
chop $maxY;
  



# read optional reference points
@refPoints = ();
$refPointCt = 0;
while ($line = <PARFILE>) {
    @fields =   split (' ', $line);
    if (@fields == 2) {
	$refPoints[$refPointCt][0] = $fields[0];
	$refPoints[$refPointCt][1] = $fields[1];
	$refPointCt ++;
    }
}



close(PARFILE);

$intervalX = ($maxX-$minX)/$binCtX;
$intervalY = ($maxY-$minY)/$binCtY;

# print "$dataFile \n";

open (INFILE,$dataFile);
$headers = <STDIN>;
chop $headers;
# print "$headers \n";

if ($useWeights) {
    open(WTFILE,$weightFile);
}
$header = <WTFILE>;

#set all bins to 0
for ($i=0; $i<$binCtX; $i++){
    for ($j=0; $j<$binCtY; $j++){
	$bin[$i][$j]=0;
    }
}


# read data and optional weights

$lineCt = 0;

while ($line = <STDIN>) {
    if ($useWeights) {
	$weight = <WTFILE>;
	chop $weight;
    } else {
	$weight = 1;
    }
    chop $line;
    @datum = split(/\t/, $line);
    $binXno = int(($datum[0]-$minX)/$intervalX);
    $binYno = int(($datum[1]-$minY)/$intervalY);
    $bin[$binXno][$binYno] += $weight;
    $lineCt++;
}


close (INFILE);


# normalize array values to percentage
# and find minimum and maximum values in array
if ($useWeights) {
    $factor = 100;
}else{
    $factor = 100/$lineCt;
}
$min = $bin[0][0]/$lineCt;
$max = $bin[0][0]/$lineCt;
$sum = 0;
for ($i = 0; $i < $binCtX; $i++) {
    for ($j = 0; $j < $binCtY; $j++) {
	$bin[$i][$j] *= $factor;
	$sum += $bin[$i][$j];
	if ($bin[$i][$j] < $min) {
	    $min = $bin[$i][$j];
	}
	if ($bin[$i][$j] > $max){
	    $max = $bin[$i][$j];
	}
    }
}

# mark reference points
for ($i=0; $i<$refPointCt; $i++){
    $binXno = int(($refPoints[$i][0]-$minX)/$intervalX);
    $binYno = int(($refPoints[$i][1]-$minY)/$intervalY);
    $bin[$binXno][$binYno] = $max;
}    


print "IGOR \n";
print "WAVES /S/N=($binCtX,$binCtY) wave_0 \n";
print "BEGIN \n";



for ($j=0; $j<$binCtX; $j++){
    for ($i=0; $i<$binCtY; $i++){
	printf "%6.3e \t", $bin[$j][$i];
    }
    print "\n";
}
print "END \n";
$xScaleMin = $minX + $intervalX/2;
$xScaleMax = $maxX - $intervalX/2;
$yScaleMin = $minY + $intervalY/2;
$yScaleMax = $maxY - $intervalY/2;
print qq(X SetScale/I x, $xScaleMin,$xScaleMax, "$xTitle", wave_0 \n);
print qq(X SetScale/I y, $yScaleMin,$yScaleMax, "$yTitle", wave_0 \n);

END
