#! /usr/users/goldnbrg/bin/perl

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# A perl script to read a table of data
# representing distances between atom pairs
# and calculate effective concentrations
# using one or more distance cutoffs
# 
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

# assumes that the input file is tab delimited
# the first line is interpretted as labels for the columnar data
# all subsequent lines are assumed to be individual data values

# The input is from stdin

# Results are written to stdout



# Effective concentration from a sample distribution of distances
# is calculated as:
# Ceff = (count/total)/(vol*Na)
# where count is the number of chains in which the two atoms
# lie within a volume element, vol, and Na is Avogadro's number.


# The volume element is a spherical shell with an inner radius
# corresponding to the sum of van der waal radii for two sulfur
# atoms (2*$r) and the outer radius is a defined cutoff radius
#    $excVol = (4/3)*$pi*(2*$r)**3;
#    $sphVol = (4/3)*$pi*$cutoff[$iCutoff]**3;
#    $shellVol = $sphVol - $excVol;

# Default value of $r is 1.55 A (from Amber library)

# optional first arguments can redifine r:
# -o : sets $r = 0
# -r : sets $r to value specified in next argument


# The script takes as additional  arguments any number of cutoff radii
# radii are assumed to have Angstrom units and 
# effective concentrations have units of Molarity


# parse arguments

if ($ARGV[0] eq "-o") {
    $r = 0;
    shift(@ARGV);
} elsif ($ARGV[0] eq "-r") {
    shift(@ARGV);
    $r = shift(@ARGV);
} else {
    $r = 1.55;
}

print "$r \n";

@cutoff = @ARGV;
$cutoffNo = @cutoff;







#constants
$Na = 6.0222e23;
$pi = atan2(1,1)*4;



#read first line of data file for column names

$headerLine = <STDIN>;
chop($headerLine);
@names = split(/\t/, $headerLine);

$colNo = @names ;



#initialize counters
for ($iCol = 0; $iCol < $colNo; $iCol++) {
    $total[$iCol] = 0;
    for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
	$count[$iCol][$iCutoff] = 0;
	}
}




# read data lines and count distances lower than cutoffs

while ($dataLine = <STDIN>) {
    @datum = split(/\t/, $dataLine);
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
        if ($datum[$iCol] =~ /\d/) {
	    $total[$iCol]++;
	    for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
		if ($datum[$iCol] <= $cutoff[$iCutoff]) {
		    $count[$iCol][$iCutoff]++;
		}
	    }
        }
    }
}

# Calculate fractions and effective concentrations

for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
    $excVol = (4/3)*$pi*(2*$r)**3;
    $sphVol = (4/3)*$pi*$cutoff[$iCutoff]**3;
    $shellVol = $sphVol - $excVol;
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	$frac[$iCol][$iCutoff] = $count[$iCol][$iCutoff]/$total[$iCol];
	$Ceff[$iCol][$iCutoff] = 1e27*$frac[$iCol][$iCutoff]/($shellVol*$Na);
	}
}

# output

# print tables of calculated values for individual cutoffs
for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
    printf "# Effective concentrations calculated \n";
    printf "# using distance cutoff of %3.1f Angstroms \n\n", $cutoff[$iCutoff];
    printf "%-12s\t%-9s\t%-9s\t%-11s\t%-11s\t%-11s\n", "Atom Pairs", "Total",
	"Count", "Fraction", "Eff. Conc. (M)";
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	printf "%11s\t", $names[$iCol];
        printf "%-5i%4s\t", $total[$iCol];
	printf "%-5i%4s\t", $count[$iCol][$iCutoff];
	printf "%8.3e%6s\t", $frac[$iCol][$iCutoff];
	printf "%8.3e%6s\n", $Ceff[$iCol][$iCutoff];
    }
    printf "\n\n";
}

#print tables of count, fract and Ceff as functions of cutoff

#count table
printf "Counts of chains with atom pairs lying within cutoff radius\n";
printf "Cutoff (A)\t";
foreach $name (@names){
    print "$name \t";
}

printf "\n";

for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
    printf "%-9.1f\t", $cutoff[$iCutoff];
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	printf "%-5i%6s\t", $count[$iCol][$iCutoff];
    }
    printf "\n";
}

printf "\n\n";

# fraction table
printf "Fraction of chains with atom pairs lying within cutoff radius\n";

printf "Cutoff (A)\t";
foreach $name (@names){
    print "$name \t";
}

printf "\n";

for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
    printf "%-9.1f\t", $cutoff[$iCutoff];
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	printf "%8.3e%6s\t", $frac[$iCol][$iCutoff];
    }
    printf "\n";
}

printf "\n\n";

# Effective concentration table
printf "Effective concentrations calculated with different cutoff radii\n";
printf "Cutoff (A)\t";
foreach $name (@names){
    print "$name \t";
}

print "\n";

for ($iCutoff = 0; $iCutoff < $cutoffNo; $iCutoff++) {
    printf "%-9.1f\t", $cutoff[$iCutoff];
    for ($iCol = 0; $iCol < $colNo; $iCol++) {
	printf "%8.3e%6s\t", $Ceff[$iCol][$iCutoff];
    }
    printf "\n";
}


    
