#! /usr/users/goldnbrg/bin/perl

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# perl script to read dg format coordinate files
# and generate input files for Tim Richmond's 
# ACCESS program for calculating accessible surface areas

# written by David P. Goldenberg 
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!

# accepts one argument, name of input file
# writes to a series of files, one file per structure
# if input file is foo.cor , outputs are foo.1.acc, foo.2.acc, etc
# if input file is foo.foo, output files are foo.foo1.acc, etc
# Output format is for a version of ACCESS modified by D.P.G. to use
# larger fields for coordinates, to allow larger structures.


$inFile = $ARGV[0];
if ($inFile =~ /\.cor$/) {
  $outName = substr($inFile, 0, -4);
} else {
  $outName = $inFile;
}

open (INFILE, $inFile);

$firstRes = 0;
while ($line = <INFILE>) {
  @inField = split(' ',$line);
  $noInFields = @inField;
  if (($noInFields == 7) && !($inField[1] =~ /^[QH]/)) {
    if ($firstRes == 0) {
      $structCt = 1;
      $firstAtom = $inField[1];
      $firstRes = $inField[2];
      open (OUTFILE, ">".$outName.".1.acc");
    } elsif (($inField[1] eq $firstAtom) && ($inField[2] == $firstRes)) {
      $structCt++;
      close (OUTFILE);
      open (OUTFILE, ">".$outName.".".$structCt.".acc"); 
    }
    $atomName = $inField[1];
    $resName = &stdResName($inField[3]);
    $resNo = $inField[2];
    $xCoor = $inField[4];
    $yCoor = $inField[5];
    $zCoor = $inField[6];

    $atomRad = &radius ($atomName,$resName);
    # Output in format to be read by modified ACCESS:
    # FRORMAT (1X,I1,6X,2A4,I5,3X,3F10.4,4X,F8.4)    
    # old format was:
    # FRORMAT (1X,I1,6X,2A4,I5,3X,3F8.4,4X,F8.4)
    # If value in first integer field is 1, surface area is not found
    # for this atom.
    
    printf OUTFILE " 0      ";
    printf OUTFILE "%-4s",  $atomName;
    printf OUTFILE "%-4s",  $resName;
    printf OUTFILE "%5d",   $resNo;
    printf OUTFILE "   ";
    printf OUTFILE "%10.4f", $xCoor;
    printf OUTFILE "%10.4f", $yCoor;
    printf OUTFILE "%10.4f", $zCoor;
    printf OUTFILE "    ";
    printf OUTFILE "%8.4f\n", $atomRad;

   
    
  }
}
close (INFILE);
close (OUTFILE);
  

sub stdResName {
  # converts funny DYANA / MOLMOL names into standard 3-letter names
  # accepts single input variable, funny name.
  # returns 3-letter name
  my ($inName);
  $inName = $_[0];
  if (length ($inName) == 3) {
    return $inName;
  } elsif ($inName eq 'CYSS') {
    return 'CYS';
  } elsif ($inName =~ /^[NC]/) {
    return substr ($inName, 1, 3);
  } elsif ($inName =~ /[\+-]$/) {
    return substr ($inName, 0, -1);
  } else {
    return substr ($inName, 1, 3);
  }
}

    
sub radius {
  # determines radius for surface area calculations
  # accepts two input variables, atom type and residue type
  # The radii used in this version are those used by
  # Chothia (1975) Nature 254, 304-308
  my ($atomType, $resType);
  $atomType = $_[0];
  $resType = $_[1];

  # carbons
  if ($atomType =~ /^C/) {
    # trigonal carbons
    if ($atomType !~ /[ABDGEZH]/) {
      return 1.76;
    } elsif ($resType eq 'ARG' && $atomType eq 'CZ') {
      return 1.76;
    } elsif ($resType eq 'ASN' && $atomType eq 'CG') {
      return 1.76;
    } elsif ($resType eq 'ASP' && $atomType eq 'CG') {
      return 1.76;
    } elsif ($resType eq 'GLN' && $atomType eq 'CD') {
      return 1.76;
    } elsif ($resType eq 'GLU' && $atomType eq 'CD') {
      return 1.76;
    } elsif ($resType eq 'PHE' && $atomType =~ /[GDEZ]/) {
      return 1.76;
    } elsif ($resType eq 'TYR' && $atomType =~ /[GDEZ]/) {
      return 1.76;
    } elsif ($resType eq 'TRP' && $atomType =~ /[GDEZH]/) {
      return 1.76;
    } else {
      #all other carbons
      return 1.87;
    }
  } elsif ($atomType =~ /^N/) { #nitrogens
    # tetrahedral N (misses alpha-amino group)
    if ($resType eq 'LYS' && $atomType eq 'NZ') {
      return 1.5;
    } else {
      return 1.65;
    }
  } elsif ($atomType =~ /^O/) { #oxygens
    return 1.40;
  } elsif ($atomType =~ /^S/) { #sulfurs
    return 1.85;
  } elsif ($atomType =~ /^P/) { #phosphorous
    return 1.85;
  } elsif ($atomType =~ /^S/) {
    return 1.85;
  #ambiguous N & O in ASN or GLN
  # for pdb files
  } elsif ($resType eq 'ASN' && $atomType =~ /^A/) {
    return 1.52
  } elsif ($resType eq 'GLN' && $atomType =~ /^A/) {
    return 1.52
  } else {
    return 0.0
  }
}

