#! /usr/users/goldnbrg/bin/perl

# NOTE: modify the previous line to point to 
# the appropriate perl for your system

# A perl script to extract distance information from dg format coordinate files
#
#       David P. Goldenberg
#       Department of Biology
#       University of Utah
#       257 South 1400 East
#       Salt Lake City, UT
#
#       goldenberg@biology.utah.edu
#
# Provided at no cost and with absolutely no guarantees!
# But, please report bugs!


# requires 3 inputs:
#  name of coordinate file
#  name of file with list of atom pairs
#  name of output file
# If the output file does not already exist, the first output line is a header identifying
# the atom pairs.  If this file exists, only data lines are output.
# Data are output as tab-delimited distances in Angstroms

# The coordinate file can contain multiple structures,
# and the program will try to figure out where each structure begins and ends
# But, it isn't very smart and can probably be confused by a badly formed file.

if (@ARGV != 3) {
    die "Incorrect number of input arguments.  Input: 
         coordinate file name, atom-pair file name, output file name \n"
}

$coorFile = shift(@ARGV);
$atomPairFile = shift(@ARGV);
$outFile = shift(@ARGV);


open (APFILE, $atomPairFile);
$noAP = &readAP(APFILE);
close (APFILE);


open (COORFILE, $coorFile);
$noStruct = &scanCoor(COORFILE);
close (COORFILE);

if (-e $outFile) {
    open(OUTFILE, ">>" . $outFile);
    &writeDistances(OUTFILE);
    close(OUTFILE);
} else {
    open(OUTFILE, ">" . $outFile);
    &writeHeader(OUTFILE);
    &writeDistances(OUTFILE);
}

sub readAP {

    #subroutine to read atom pairs from an already opened file
    #accepts one input variable, a string with the file handle
    #assumes that atoms are listed as pairs 
    #with each line containing first atom name, first residue number
    #second atom name and second residue number, all space delimited
    #lines begining with "#" are ignored
    #Returns no. of atom pairs, determined as the no. of elements in
    #@atom1
    my ($filehandle, $line, @field, $noAP);
    $filehandle = $_[0];
    @atom1 = ();
    @frag1 = ();
    @atom2 = ();
    @frag2 = ();
    while ($line = <$filehandle>) {
	if (!($line =~ /^#/)) {
	      @field = split (' ',$line);
	      @atom1 = (@atom1, $field[0]);
	      @frag1 = (@frag1, $field[1]);
	      @atom2 = (@atom2, $field[2]);
	      @frag2 = (@frag2, $field[3]);
	}
    }
    $noAP = @atom1;
}     
	  

sub scanCoor {
    #subroutine reads opened dg format coordinate file 
    #and scans for atom pairs defined in global variable arrays
    #@atom1, @frag1, @atom2, @frag3
    #The subroutine looks for multiple coordinate sets in the file,
    #but it isn't very smart.  If there are incomplete coordinate
    #sets, something bad will probably happen!
    #For each atom pair, distance is calculated for each structure
    #in the coordinate file.
    #Distances are stored in the global two-dimensional array
    #@dist[][].  The first index is the atom-pair and
    #the secod index is the structure number.
    #Accepts one input variable, string representing the file handle
    #Returns the number of structures in the coordinate file,
    #calculated as the number of times the first specified
    #atom pair is found

    my ($filehandle, @field, $line, $noFields, $iatom, $dist);
    my (@atom1found, @atom2found, @count, $iap, $noStruct);
    $filehandle = $_[0];
    for ($iap = 0; $iap < $noAP; $iap++) {
	$atom1found[$iap] = 0;
	$atom2found[$iap] = 0;
	$count[$iap] = 0;
	@dist[$iap] = ();
    }
    while ($line = <$filehandle>) {
	@field = split(' ', $line);
	$noFields = @field;
	if ($noFields == 7) {
	    for ($iap = 0; $iap < @atom1; $iap++) {
		if ($field[1] eq $atom1[$iap] && $field[2] == $frag1[$iap]) {
		    $X1[$iap] = $field[4];
		    $Y1[$iap] = $field[5];
		    $Z1[$iap] = $field[6];
		    $atom1found[$iap] = 1;
		} elsif ($field[1] eq $atom2[$iap] && $field[2] == $frag2[$iap]) {
		    $X2[$iap] = $field[4];
		    $Y2[$iap] = $field[5];
		    $Z2[$iap] = $field[6];
		    $atom2found[$iap] = 1;
		}
		if ($atom1found[$iap] == 1 && $atom2found[$iap] == 1) {
		    $dist[$iap][$count[$iap]] = &distance
			($X1[$iap],$Y1[$iap],$Z1[$iap],
			 $X2[$iap],$Y2[$iap],$Z2[$iap]);
		    $count[$iap]++;
		    $atom1found[$iap] = 0;
		    $atom2found[$iap] = 0;
		}
	    }
	}
    }
    $noStruct = $count[0]
}

	
sub distance {
    $sumSq = ($_[0] - $_[3]) * ($_[0] - $_[3]);
    $sumSq +=  ($_[1] - $_[4]) * ($_[1] - $_[4]);
    $sumSq +=  ($_[2] - $_[5]) * ($_[2] - $_[5]);
    return $sumSq**0.5;
}



sub writeDistances {
    my ($iap,$struct,$filehandle);
    $filehandle = $_[0];
    for ($struct = 0; $struct < $noStruct; $struct++) {
	for ($iap = 0; $iap < $noAP; $iap++) {
	    printf $filehandle "%-5.3f%6s\t",$dist[$iap][$struct],"";
	}
	print $filehandle "\n";
    }
}

sub writeHeader {
    my ($iap, $filehandle);
    $filehandle = $_[0];
    for ($iap = 0; $iap < $noAP; $iap++) {
	print $filehandle "$atom1[$iap] $frag1[$iap] - $atom2[$iap] $frag2[$iap]\t";
    }
    print $filehandle "\n";
}
